library IEEE;
use IEEE.std_logic_1164.all;


entity processor is
  generic (

    
    C_OPB_AWIDTH : integer := 32;
    C_OPB_DWIDTH : integer := 32;
    C_BASEADDR   : std_logic_vector := X"0FEF_0000";   
    C_HIGHADDR   : std_logic_vector := X"0FEF_3FFF");  

  
  
  port (
    -- Global signals
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;

    -- OPB signals
    OPB_ABus    : in std_logic_vector (31 downto 0);
    OPB_BE      : in std_logic_vector (3 downto 0);
    OPB_DBus    : in std_logic_vector (31 downto 0);
    OPB_RNW     : in std_logic;
    OPB_select  : in std_logic;
    OPB_seqAddr : in std_logic;

    -- Slave signals
    P_DBus    : out std_logic_vector (31 downto 0);
    P_errAck  : out std_logic;
    P_retry   : out std_logic;
    P_toutSup : out std_logic;
    P_xferAck : out std_logic
    );
end processor;

architecture structural of processor is


signal AT:std_logic_vector(31 downto 0) := X"0000016F";
signal RT:std_logic_vector(31 downto 0) := X"00000002";
signal counter: std_logic_vector(31 downto 0) := X"00000000";
signal start : std_logic := '0';

signal data_bus_ce : std_logic_vector(31 downto 0);
signal raddr : std_logic_vector(15 downto 0);


--multiplier variables:

signal Ra_R : std_logic_vector(31 downto 0);
signal Ra_A : std_logic_vector(31 downto 0);
signal La_R : std_logic_vector(31 downto 0);
signal La_A : std_logic_vector(31 downto 0);

signal LprodA : std_logic_vector(63 downto 0);
signal LprodR : std_logic_vector(63 downto 0);
signal RprodA : std_logic_vector(63 downto 0);
signal RprodR : std_logic_vector(63 downto 0);  



-- Signals used when reading from block
-- ram and filling status register
signal cs : std_logic;
signal ce : std_logic;
signal rnw : std_logic;
signal xfer : std_logic;


-- Dummy signals. Reserved for future enhancements
-- We currently not write from microblze (XIo_Out)
signal wdata : std_logic_vector (31 downto 0);
signal be    : std_logic_vector (3 downto 0);



--component mul32c Port (
--  a: in std_logic_vector (31 downto 0);
--  b: in std_logic_vector (31 downto 0);
--  prod: out std_logic_vector (63 downto 0)
--  );

--end component;

begin
--Each peak detector requires two multiplications so we'll do them in parallel

--LeftPeakA : mul32c port map(
--  a => La_A,
 -- b => AT,
  --prod => LprodA
 -- );

--LeftPeakR : mul32c port map(
  --a => La_R,
  --b =>RT,
  --prod => LprodR
 -- );

--RightPeakA : mul32c port map(
  --a => Ra_A,
  --b => AT,
  --prod => RprodA
 -- );
--RightPeakR : mul32c port map(
  --a => Ra_R,
  --b => RT,
  --prod => RprodR
  --);



-- Check to see if this device is being addressed
cs <= OPB_select when OPB_ABus(31 downto 16) = X"0FEF" else '0';

--raddr stores the address of register we are trying to access
process (OPB_Clk)
begin
  if OPB_Clk'event and OPB_Clk = '1' then
    if OPB_RST = '1' then
      raddr <= "0000000000000000";
    else
      raddr <= OPB_ABus(15 downto 0);
    end if;
  end if;
end process;


-- Latching RNW signal
process (OPB_Clk)
begin
  if OPB_Clk'event and OPB_Clk = '1' then
    if OPB_Rst = '1' then
      rnw <= '0';
    else
      rnw <= OPB_RNW;
    end if;
  end if;
end process;


-- Latching BE signal (byte enable). Dummy signal
process (OPB_Clk)
begin
  if OPB_Clk'event and OPB_Clk = '1' then
    if OPB_Rst = '1' then
      be <= "0000";
    else
      be <= OPB_BE;
    end if;
  end if;
end process;


-- The following process is dummy. It is used to
-- create a mux between this entity and OPB_DBus
process (OPB_Clk)
begin
  if OPB_Clk'event and OPB_Clk = '1' then
    if OPB_Rst = '1' then
      wdata <= X"0000_0000";
    else
      wdata <= OPB_DBus; 
    end if;
  end if;   
end process;



-- Reset the counter every time chip is selected
--
process(OPB_Clk,cs,start)
  begin
     
    if OPB_Clk'event and OPB_Clk='1' then
      if cs='0' then
        start <='0';
     end if;

     if cs='1' and start='0' then
        start<='1';
        counter<=X"00000000";
     -- else
    --   counter<=counter+1;

      end if;
      
    end if;  
 end process;
    


--
-- write
-- 
process (OPB_Clk, OPB_Rst,rnw)
begin
  if OPB_Rst='1' then
    Ra_R <= X"00000000"; 
  elsif OPB_Clk'event and OPB_Clk='1' and not rnw ='1' then

    Ra_R <= wdata(31 downto 0);
  
  end if;
end process;
   

process (OPB_Clk, OPB_Rst)
begin
  if OPB_Rst='1' then
    data_bus_ce <= X"00000000"; 
  elsif OPB_Clk'event and OPB_Clk='1' then

    case raddr(7 downto 0) is
       when X"00" =>  data_bus_ce <= X"FEFE_FEFE";
       when X"04" =>  data_bus_ce <= X"ABCD_ABCD";
       when X"08" =>  data_bus_ce <= X"FAFA_FAFA";
       when X"0c" =>  data_bus_ce <= X"CACA_CACA";
      when others =>  data_bus_ce <= X"c0b0_b0c0";
    end case;

  end if;
end process;


-- XFER is transfer acknowledge
xfer <= cs;                              --change this to delay until read is done

-- Slave data bus
P_DBus(31 downto 0) <= data_bus_ce when (cs = '1') else X"0000_0000";

-- Tie unused signals to zero
P_errAck <= '0';
P_retry <= '0';
P_toutSup <= '0';

P_xferAck <= xfer;

end structural;
